"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyFiles = void 0;
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const applyFiles = async (files, binaries, basePath = './') => {
    const errors = [];
    await promises_1.default.mkdir(basePath, { recursive: true });
    const baseResolvedPath = path_1.default.resolve(basePath);
    for (const file of files) {
        const filePath = path_1.default.join(basePath, file.path);
        const resolvedPath = path_1.default.resolve(filePath);
        try {
            if (!resolvedPath.startsWith(baseResolvedPath)) {
                throw new Error(`Path is out of the app folder`);
            }
            if (!file.type || file.type === 'file') {
                const dirPath = path_1.default.dirname(filePath);
                await promises_1.default.mkdir(dirPath, { recursive: true });
                if (file.contents) {
                    await promises_1.default.writeFile(filePath, file.contents, file.encoding || 'utf-8');
                }
                else if (typeof file.binary === 'number') {
                    const binary = binaries[file.binary];
                    if (binary) {
                        await promises_1.default.copyFile(binary.path, filePath);
                    }
                    else {
                        throw new Error(`Binary not found at index ${file.binary}`);
                    }
                }
            }
            else if (file.type === 'directory') {
                await promises_1.default.mkdir(filePath, { recursive: true });
            }
            else if (file.type === 'delete') {
                await promises_1.default.rm(filePath, { recursive: true, force: true });
            }
        }
        catch (error) {
            errors.push({
                error,
                path: file.path,
            });
            console.error('Could not write file', filePath, error);
        }
    }
    return { errors };
};
exports.applyFiles = applyFiles;
