"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = filesRouter;
const promises_1 = __importDefault(require("fs/promises"));
const os_1 = __importDefault(require("os"));
const path_1 = __importDefault(require("path"));
const express_1 = __importDefault(require("express"));
const multer_1 = __importDefault(require("multer"));
const config_js_1 = require("../config.js");
const files_js_1 = require("./files.js");
async function filesRouter() {
    const router = express_1.default.Router();
    const upload = (0, multer_1.default)({
        dest: await promises_1.default.mkdtemp(path_1.default.join(os_1.default.tmpdir(), 'plugin-dev-server')),
    });
    router.put('/', express_1.default.json(), upload.array('binaries'), async (req, res) => {
        try {
            let { files } = req.body;
            if (typeof files === 'string') {
                // multipart send files as string
                files = JSON.parse(files);
            }
            const binaries = req.files;
            console.log('Applying files update of size', files.length);
            const { errors } = await (0, files_js_1.applyFiles)(files, binaries, config_js_1.DEFAULT_APP_FOLDER);
            const errorsJson = errors.map((errorObj) => {
                var _a;
                return ({
                    ...errorObj,
                    error: (_a = errorObj.error) === null || _a === void 0 ? void 0 : _a.message,
                });
            });
            res.status(200).send({ status: 'ok', errors: errorsJson });
        }
        catch (error) {
            console.error('Error updating files', error);
            res.status(500).send({ error });
        }
    });
    return router;
}
