"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.startLogsServer = void 0;
const ws_1 = require("ws");
const config_js_1 = require("./config.js");
const sendDataToSocket = (ws, type, pid, output) => {
    ws.send(JSON.stringify({
        type,
        id: pid,
        data: output,
    }));
};
const startLogsServer = ({ server }) => {
    const wss = new ws_1.WebSocketServer({ noServer: true });
    const processes = {};
    const processClientsMap = new Map();
    const clientProcessMap = new Map();
    const registerProcessForLogStreaming = (process) => {
        processes[process.pid] = process;
        process.all.on('data', (chunk) => {
            var _a;
            const chunkStr = chunk.toString();
            (_a = processClientsMap.get(process.pid)) === null || _a === void 0 ? void 0 : _a.forEach((client) => {
                sendDataToSocket(client, 'output', process.pid, chunkStr);
            });
        });
        process.on('close', () => {
            var _a;
            (_a = processClientsMap.get(process.pid)) === null || _a === void 0 ? void 0 : _a.forEach((client) => {
                var _a;
                sendDataToSocket(client, 'exit', process.pid);
                (_a = clientProcessMap.get(client)) === null || _a === void 0 ? void 0 : _a.delete(process.pid);
            });
            processClientsMap.delete(process.pid);
        });
    };
    wss.on('connection', function connection(ws) {
        console.log('Connected client to logs socket');
        ws.on('error', (err) => {
            console.error('Encountered error with logs socket', err);
        });
        ws.on('close', () => {
            var _a;
            (_a = clientProcessMap.get(ws)) === null || _a === void 0 ? void 0 : _a.forEach((pid) => {
                var _a;
                (_a = processClientsMap.get(pid)) === null || _a === void 0 ? void 0 : _a.delete(ws);
            });
            clientProcessMap.delete(ws);
        });
        ws.on('message', (data) => {
            var _a, _b, _c;
            try {
                const message = JSON.parse(data);
                if (message.type === 'stream') {
                    const pid = message.id;
                    if (!pid) {
                        return;
                    }
                    if (message.sendPrevious) {
                        const existingOutput = (_a = processes[pid]) === null || _a === void 0 ? void 0 : _a.allStr;
                        if (existingOutput) {
                            sendDataToSocket(ws, 'output', pid, existingOutput);
                        }
                    }
                    if (!processClientsMap.get(pid)) {
                        processClientsMap.set(pid, new Set());
                    }
                    processClientsMap.get(pid).add(ws);
                    if (!clientProcessMap.get(ws)) {
                        clientProcessMap.set(ws, new Set());
                    }
                    clientProcessMap.get(ws).add(pid);
                }
                else if (message.type === 'stop') {
                    const pid = message.id;
                    if (!pid) {
                        return;
                    }
                    (_b = processClientsMap.get(pid)) === null || _b === void 0 ? void 0 : _b.delete(ws);
                    (_c = clientProcessMap.get(ws)) === null || _c === void 0 ? void 0 : _c.delete(pid);
                }
            }
            catch (error) {
                console.error('Could not decode message', error, data);
            }
        });
    });
    const LISTENS_PATHS = new Set(['/logs', `${config_js_1.API_PATH}/logs`]);
    server.on('upgrade', function upgrade(request, socket, head) {
        const { pathname } = new URL(request.url, 'wss://base.url');
        // due to redirection of the netlify-cli, req.url might have .ntlfy-dev in it as the first part
        if (LISTENS_PATHS.has(pathname)) {
            wss.handleUpgrade(request, socket, head, function done(ws) {
                wss.emit('connection', ws, request);
            });
        }
        else {
            socket.destroy();
        }
    });
    return {
        registerProcessForLogStreaming,
    };
};
exports.startLogsServer = startLogsServer;
